package argumentrelevance.application;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

import argumentrelevance.argumentranker.AbstractRanker;
import argumentrelevance.argumentranker.FrequencyAvgRanker;
import argumentrelevance.argumentranker.FrequencyMaxRanker;
import argumentrelevance.argumentranker.FrequencyMinRanker;
import argumentrelevance.argumentranker.FrequencySumRanker;
import argumentrelevance.argumentranker.JaccardAvgRanker;
import argumentrelevance.argumentranker.JaccardMaxRanker;
import argumentrelevance.argumentranker.JaccardMinRanker;
import argumentrelevance.argumentranker.JaccardSumRanker;
import argumentrelevance.argumentranker.MostPremisesRanker;
import argumentrelevance.argumentranker.PageRankAvgRanker;
import argumentrelevance.argumentranker.PageRankMaxRanker;
import argumentrelevance.argumentranker.PageRankMinRanker;
import argumentrelevance.argumentranker.PageRankSumRanker;
import argumentrelevance.argumentranker.RandomRanker;
import argumentrelevance.argumentranker.SentiwordnetAvgRanker;
import argumentrelevance.argumentranker.SentiwordnetMaxRanker;
import argumentrelevance.argumentranker.SentiwordnetMinRanker;
import argumentrelevance.argumentranker.SentiwordnetSumRanker;
import argumentrelevance.rankingcorrelation.AbstractCorrelationComputer;
import argumentrelevance.rankingcorrelation.KendallCorrelationComputer;
import argumentrelevance.rankingcorrelation.PearsonCorrelationComputer;
import argumentrelevance.storage.Argument;
import argumentrelevance.storage.ArgumentRanking;
import argumentrelevance.storage.ArgumentUnit;
import argumentrelevance.storage.ArgumentUnitSource;
import argumentrelevance.storage.CSVLoader;

/**
 * This executable class is the main class for reproducing the ground-truth
 * experiments in the argument relevance paper.
 * 
 * The class computes argument rankings for each conclusion in the benchmark
 * dataset and compares them with the benchmark rankings. The results are
 * printed to the console.
 * 
 * @author henning.wachsmuth
 *
 */
public class RankingOfArguments {

	// -------------------------------------------------------------------------
	// I/O PARAMETERS
	// -------------------------------------------------------------------------
	
	/**
	 * The path of the CSV file that stores the source information for all 
	 * argument units from the argument unit graph. The columns of the file are 
	 * as follows:
	 * 
	 * - Global Argument Unit ID
	 * - Corpus name
	 * - AIFdb Unit ID corpus
	 * - AIFdb Map ID
	 * 
	 * (one entry for each AIFDB node)
	 */
	private static final String UNIT_SOURCE_LIST_PATH = 
			"./data/argument-relevance/unit-source-list.csv";
	
	/**
	 * The path of the CSV file that stores all information for each argument
	 * unit. The columns of the file are as follows:
	 * 
	 * - Global Argument Unit ID (same as in unit source list)
	 * - Conclusion Text
	 * - Frequency (of the conclusion text in the AIFdb corpus)
	 * - PageRank
	 * 
	 * (one entry for each conclusion text)
	 */
	private static final String ARGUMENT_UNIT_LIST_PATH = 
			"./data/argument-relevance/argument-unit-list.csv";
	
	/**
	 * The path of the CSV file that stores all information for each argument.
	 * The columns of the file are as follows:
	 * 
	 * - Global argument ID
	 * - Global argument unit ID of conclusion (same as in argument unit list)
	 * - Conclusion text  
	 * - Number of premises
	 * - Global argument unit ID of premise 1 (same as in argument unit list)
	 * - Premise 1 text  
	 * - ...   (last two repeated for all other premises)
	 * 
	 * (one entry for each argument)
	 */
	private static final String ARGUMENT_LIST_PATH = 
			"./data/argument-relevance/argument-list.csv";
	
	/**
	 * The path of the CSV file that stores the ground-truth argument rankings.
	 * The columns of the file are as follows:
	 * 
	 * - Argument ID (same as in argument list)
	 * - Aggregate rank
	 * - Mean rank
	 * - Rank of annotator #1
	 * - Rank of annotator #2
	 * - Rank of annotator #3
	 * - Rank of annotator #4
	 * - Rank of annotator #5
	 * - Rank of annotator #6
	 * 
	 * - Conclusion argument unit ID (same as in argument unit list)
	 * - Conclusion text
	 * - Number of premises
	 * - Corrected conlusion text
	 * - Corrected premises text
	 * - Premise argument unit ID 1 (same as in argument unit list)
	 * - Premise text 1  
	 * - ...   (last two repeated for all other premises)
	 * 
	 */
	private static final String GROUND_TRUTH_LIST_PATH = 
			"./data/argument-relevance/ground-truth-list.csv";
	
	/**
	 * The number of annotators whose rankings are in the ground-truth list.
	 * Needs to be specified in order to ensure that the ground-truth data can
	 * be read correctly.
	 */
	private static final int NUMBER_OF_ANNOTATORS = 7;
	
	/**
	 * The index of the ground-truth rankings to be used for evaluation. The 
	 * first rankings are the aggregate rankings, the second the mean rankings.
	 * All remaining refer to the rankings of the different annotators.
	 */
	private static final int INDEX_OF_GROUND_TRUTH_RANKINGS = 0;
	
	/**
	 * The names and rankers of all considered approaches
	 */
	private static final Object [][] APPROACHES = {
		{"PageRank min", new PageRankMinRanker()},
		{"PageRank avg", new PageRankAvgRanker()},
		{"PageRank max", new PageRankMaxRanker()},
		{"PageRank sum", new PageRankSumRanker()},
	};
	
	/**
	 * The names and rankers of all considered approaches
	 */
	private static final Object [][] BASELINES = {
		{"Frequency min", new FrequencyMinRanker()},
		{"Frequency avg", new FrequencyAvgRanker()},
		{"Frequency max", new FrequencyMaxRanker()},
		{"Frequency sum", new FrequencySumRanker()},
		
		{"Similarity min", new JaccardMinRanker()},
		{"Similarity avg", new JaccardAvgRanker()},
		{"Similarity max", new JaccardMaxRanker()},
		{"Similarity sum", new JaccardSumRanker()},
		
		{"Sentiment min", new SentiwordnetMinRanker()},
		{"Sentiment avg", new SentiwordnetAvgRanker()},
		{"Sentiment max", new SentiwordnetMaxRanker()},
		{"Sentiment sum", new SentiwordnetSumRanker()},

		{"Most premises", new MostPremisesRanker()},
		{"Random ranks", new RandomRanker()},
	};
	
	/**
	 * What correlation type to use. Notice that Pearson requires that all 
	 * rankings are non-trivial, i.e., have at least two different ranks
	 */
	private static final String CORRELATION_TYPE = "Kendall";
	
	/**
	 * The number of decimal places in the printed output
	 */
	protected static int DECIMAL_PLACES = 2;
	
	
	// -------------------------------------------------------------------------
	// INPUT STORAGE
	// -------------------------------------------------------------------------
	
	/**
	 * The map that stores all argument units. The key is the global argument 
	 * unit ID.
	 */
	private Map<Integer, ArgumentUnit> argumentUnits;
	
	/**
	 * The map that stores all arguments. The key is the global argument ID.
	 */
	private Map<Integer, Argument> arguments;
	
	/**
	 * The map that stores all ground-truth argument rankings for each given
	 * conclusion. The key is the global argument unit ID of the conclusion a 
	 * ranking refers to.
	 */
	private Map<Integer, List<ArgumentRanking>> groundTruthRankings = 
			new HashMap<Integer, List<ArgumentRanking>>();
		
	
	
	// -------------------------------------------------------------------------
	// STORAGE
	// -------------------------------------------------------------------------
	
	/**
	 * Storage for the arguments ranking for each given conclusion, one map
	 * for each considered approach
	 */
	private List<Map<Integer, ArgumentRanking>> approachRankings;
	
	/**
	 * Storage for the arguments ranking for each given conclusion, one map
	 * for each considered baseline
	 */
	private List<Map<Integer, ArgumentRanking>> baselineRankings;
	
	/**
	 * Storage for the correlation between each ground-truth ranking and the 
	 * respective ranking computed one of the approaches, one map for each
	 * approach.
	 */
	private List<TreeMap<Integer, Double>> approachCorrelations;
	
	/**
	 * Storage for the correlation between each ground-truth ranking and the 
	 * respective ranking computed one of the baselines, one map for each
	 * baseline.
	 */
	private List<TreeMap<Integer, Double>> baselineCorrelations;
	
	/**
	 * All results derived from the computed correlations for each approach,
	 * one list per approach.
	 * 
	 * - Entry 0: Average correlation
	 * - Entry 1: Number of best correlations
	 * - Entry 2: Number of worst correlations
	 */
	private List<List<Double>> approachResults;
	
	/**
	 * All results derived from the computed correlations for each baseline,
	 * one list per baseline.
	 * 
	 * - Entry 0: Average correlation
	 * - Entry 1: Number of best correlations
	 * - Entry 2: Number of worst correlations
	 */
	private List<List<Double>> baselineResults;
	
	// -------------------------------------------------------------------------
	// OBJECTS
	// -------------------------------------------------------------------------
	
	/**
	 * The correlation computation to be used
	 */
	private AbstractCorrelationComputer correlator;
	
	
	
	// -------------------------------------------------------------------------
	// INITIALIZATION
	// -------------------------------------------------------------------------
	
	/**
	 * Initializes the storage
	 */
	public RankingOfArguments(){
		this.approachRankings = new ArrayList<Map<Integer, ArgumentRanking>>();
		this.approachCorrelations = new ArrayList<TreeMap<Integer, Double>>();
		this.approachResults = new ArrayList<List<Double>>();
		this.baselineRankings = new ArrayList<Map<Integer, ArgumentRanking>>();
		this.baselineCorrelations = new ArrayList<TreeMap<Integer, Double>>();
		this.baselineResults = new ArrayList<List<Double>>();
		
		this.correlator = null;
		if ("Kendall".equals(CORRELATION_TYPE)){
			correlator = new KendallCorrelationComputer();
		} else if ("Pearson".equals(CORRELATION_TYPE)){
			correlator = new PearsonCorrelationComputer();
		}
	}
	
	// -------------------------------------------------------------------------
	// PROCESSING
	// -------------------------------------------------------------------------

	/**
	 * The method that runs the whole process of ranking arguments.
	 */
	private void run(){
		this.loadAllInputData();
		this.computeAllArgumentRankings();
		this.computeAllRankCorrelations();
		this.computeAllAggregateResults();
		this.printResultsToScreen();
	}
	
	/**
	 * Loads all input data.
	 */
	private void loadAllInputData(){
		CSVLoader csvLoader = new CSVLoader();
		Map<Integer, List<ArgumentUnitSource>> unitSourceMap = 
				csvLoader.loadUnitSourceMap(UNIT_SOURCE_LIST_PATH);
		this.argumentUnits = csvLoader.loadArgumentUnitMap(
				ARGUMENT_UNIT_LIST_PATH, unitSourceMap);
		this.arguments = csvLoader.loadArgumentMap(ARGUMENT_LIST_PATH);
		this.groundTruthRankings = csvLoader.loadAllGroundTruthRankings(
				GROUND_TRUTH_LIST_PATH, NUMBER_OF_ANNOTATORS);
	}
	
	/**
	 * Computes arguments rankings for all conclusions in the ground-truth
	 * ranking data, once for each PageRank approach and each baseline.
	 */
	private void computeAllArgumentRankings(){
		for (int i=0; i<APPROACHES.length; i++) {
			this.approachRankings.add(
					((AbstractRanker) APPROACHES[i][1]).computeAllRankings(
							this.groundTruthRankings, this.argumentUnits, 
							this.arguments));
		}

		for (int i=0; i<BASELINES.length; i++) {
			this.baselineRankings.add(
					((AbstractRanker) BASELINES[i][1]).computeAllRankings(
							this.groundTruthRankings, this.argumentUnits, 
							this.arguments));
		}
	}
	
	/**
	 * Computes all rank correlations between the ground-truth rankings
	 * and each of the computed rankings.
	 */
	private void computeAllRankCorrelations(){
		// Get chosen ground-truth ranking
		Map<Integer, ArgumentRanking> chosenGroundTruthRankings = 
				new HashMap<Integer, ArgumentRanking>();
		for (int conclusionID : this.groundTruthRankings.keySet()) {
			chosenGroundTruthRankings.put(conclusionID, 
					this.groundTruthRankings.get(conclusionID).get(
							INDEX_OF_GROUND_TRUTH_RANKINGS));
		}

		for (int i=0; i<APPROACHES.length; i++) {
			this.approachCorrelations.add(
					this.correlator.computeAllCorrelations(
							this.approachRankings.get(i), 
							chosenGroundTruthRankings));
		}
		
		for (int i=0; i<BASELINES.length; i++) {
			this.baselineCorrelations.add(
					this.correlator.computeAllCorrelations(
							this.baselineRankings.get(i), 
							chosenGroundTruthRankings));
		}
	}
	
	/**
	 * Computes all aggregate results from the previously computer correlations.
	 */
	private void computeAllAggregateResults(){
		for (int i=0; i<APPROACHES.length; i++) {
			List<Double> results = new ArrayList<Double>();
			results.add(this.correlator.computeAverageCorrelation(
					this.approachCorrelations.get(i)));
			results.add(0.0);
			results.add(0.0);
			this.approachResults.add(results);
		}
		
		for (int i=0; i<BASELINES.length; i++) {
			List<Double> results = new ArrayList<Double>();
			results.add(this.correlator.computeAverageCorrelation(
					this.baselineCorrelations.get(i)));
			results.add(0.0);
			results.add(0.0);
			this.baselineResults.add(results);
		}
		
		for (int conclusionID : this.groundTruthRankings.keySet()) {
			// Determine min and max correlation for current conclusion
			double min = 1.1;
			double max = -1.1;
			for (Map<Integer,Double> correlations : this.approachCorrelations) {
				if (correlations.get(conclusionID) > max){
					max = correlations.get(conclusionID);
				}
				if (correlations.get(conclusionID) < min){
					min = correlations.get(conclusionID);
				}
			}
			for (Map<Integer,Double> correlations : this.baselineCorrelations) {
				if (correlations.get(conclusionID) > max){
					max = correlations.get(conclusionID);
				}
				if (correlations.get(conclusionID) < min){
					min = correlations.get(conclusionID);
				}
			}
			// Increase best and worst counters 
			for (int i=0; i<this.approachCorrelations.size(); i++) {
				if (Double.compare(this.approachCorrelations.get(i).get(
						conclusionID), max) == 0){
					this.approachResults.get(i).set(1, 
							this.approachResults.get(i).get(1)+1);
				}
				if (Double.compare(this.approachCorrelations.get(i).get(
						conclusionID), min) == 0){
					this.approachResults.get(i).set(2, 
							this.approachResults.get(i).get(2)+1);
				}
			}
			for (int i=0; i<this.baselineCorrelations.size(); i++) {
				if (Double.compare(this.baselineCorrelations.get(i).get(
						conclusionID), max) == 0){
					this.baselineResults.get(i).set(1, 
							this.baselineResults.get(i).get(1)+1);
				}
				if (Double.compare(this.baselineCorrelations.get(i).get(
						conclusionID), min) == 0){
					this.baselineResults.get(i).set(2, 
							this.baselineResults.get(i).get(2)+1);
				}
			}
		}
	}
	
	
	/**
	 * Prints all results to screen
	 */
	private void printResultsToScreen(){
		System.out.println("\n");
		System.out.println(CORRELATION_TYPE + " correlation");
		for (int i=0; i< 300; i++){
			System.out.print("=");
		}
		System.out.println();
		
		System.out.print("Ranking approach  #Best #Worst  " + CORRELATION_TYPE + 
				" \t");
		for (int conclusionID : approachCorrelations.get(0).keySet()) {
			System.out.print("c-" + conclusionID + "\t");
		}
		System.out.println();
		
		for (int i=0; i< 300; i++){
			System.out.print("=");
		}
		System.out.println();
		
		for (int i=0; i<APPROACHES.length; i++){
			System.out.print(APPROACHES[i][0] + "\t  " + 
					(int) this.approachResults.get(i).get(1).doubleValue()+"\t"+ 
					(int) this.approachResults.get(i).get(2).doubleValue()+"\t"+ 
					this.roundValue(this.approachResults.get(i).get(0))+"\t\t");
			for (int conclusionID : approachCorrelations.get(i).keySet()) {
				System.out.print(
						this.roundValue(approachCorrelations.get(i).get(
								conclusionID)) + "\t");
			}
			System.out.println();
		}
		
		for (int i=0; i< 300; i++){
			System.out.print("-");
		}
		System.out.println();
		
		for (int i=0; i<BASELINES.length; i++){
			System.out.print(BASELINES[i][0] + "\t  " + 
					(int) this.baselineResults.get(i).get(1).doubleValue()+"\t"+ 
					(int) this.baselineResults.get(i).get(2).doubleValue()+"\t"+ 
					this.roundValue(this.baselineResults.get(i).get(0))+"\t\t");
			for (int conclusionID : baselineCorrelations.get(i).keySet()) {
				System.out.print(
						this.roundValue(baselineCorrelations.get(i).get(
								conclusionID)) + "\t");
			}
			System.out.println();
		}
		
		for (int i=0; i< 300; i++){
			System.out.print("=");
		}
		System.out.println();
	}
	
	/**
	 * Rounds the given value to a number with the globally specified number of 
	 * decimal places.
	 * 
	 * @param value The value
	 * @return The rounded value
	 */
	private double roundValue(double value){
		if (DECIMAL_PLACES == 0){
			return (int) Math.round(value);
		}
		double rounder = 1.0;
		for (int i=0; i<DECIMAL_PLACES; i++)
			rounder *= 10.0;
		return Math.round(value*rounder)/rounder;
	}
	
	
	
	// -------------------------------------------------------------------------
	// MAIN METHOD
	// -------------------------------------------------------------------------

	/**
	 * Runs the whole application.
	 * 
	 * @param args Not used.
	 */
	public static void main (String [] args){
		RankingOfArguments application = new RankingOfArguments();
		application.run();
	}	
}