package argumentrelevance.argumentranker;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import argumentrelevance.storage.ArgumentUnit;

/**
 * This class is the abstract superclass of all evaluated ranking approaches
 * that compute an argument's score based on the Jaccard similarity between
 * an argument's premises and its conclusion.
 * 
 * @author henning.wachsmuth
 *
 */
public abstract class AbstractJaccardBasedRanker 
extends AbstractSimilarityBasedRanker {

	@Override
	public List<Double> computeSimilarities(List<ArgumentUnit> premises,
			ArgumentUnit conclusion) {
		List<Double> similarities = new ArrayList<Double>();
		
		Map<String, Integer> conclusionWords = 
				this.getWords(conclusion.getText());
		for (ArgumentUnit premise : premises) {
			Map<String, Integer> premiseWords = 
					this.getWords(premise.getText());
			similarities.add(
					this.getJaccardSimilarity(conclusionWords, premiseWords));
		}
		return similarities;
	}	
	
	/**
	 * Returns counts of all words contained in the given text span.
	 * 
	 * @param textSpan The text span
	 * @return The counts
	 */
	private Map<String, Integer> getWords(String textSpan){
		// Not case-sensitive
		textSpan = textSpan.toLowerCase();
		
		// Delete common non-word characters
		textSpan = textSpan.replace(",", "");
		textSpan = textSpan.replace(".", "");
		textSpan = textSpan.replace("!", "");
		textSpan = textSpan.replace("?", "");
		textSpan = textSpan.replace(":", "");
		textSpan = textSpan.replace(";", "");
		
		// Then split into words and count
		Map<String, Integer> counts = new HashMap<String, Integer>();
		for (String word : textSpan.split(" |\t|\n")) {
			if (!counts.containsKey(word)){
				counts.put(word, 0);
			}
			counts.put(word, counts.get(word)+1);
		}
		return counts;
	}
	
	/**
	 * Computes and returns the Jaccard similarity between the vectors 
	 * represented by the two given word counts.
	 * 
	 * @param wordsA Words with counts a
	 * @param wordsB Words with counts b
	 * @return The Jaccard similarity
	 */
	private double getJaccardSimilarity(Map<String, Integer> wordsA, 
			Map<String, Integer> wordsB){
		double intersect = 0.0;
		for (String word : wordsA.keySet()) {
			if (wordsB.containsKey(word)){
				intersect += 1.0;
			}
		}
		return intersect / (wordsA.size() + wordsB.size() - intersect);
	}
}









