package argumentrelevance.argumentranker;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

import argumentrelevance.storage.Argument;
import argumentrelevance.storage.ArgumentRanking;
import argumentrelevance.storage.ArgumentUnit;

/**
 * This class is the abstract superclass of all evaluated ranking approaches
 * that compute an argument's score solely based on the argument's premises.
 * 
 * @author henning.wachsmuth
 *
 */
public abstract class AbstractPremiseBasedRanker 
extends AbstractRanker {
	
	@Override
	public ArgumentRanking computeArgumentScores(
			ArgumentUnit conclusion, List<Argument> arguments,
			Map<Integer, ArgumentUnit> allArgumentUnits,
			Map<Integer, Argument> allArguments) {
		// Compute and return one score for each argument
		TreeMap<Integer, Double> argID2Score = new TreeMap<Integer, Double>();
		for (Argument argument : arguments) {
			// Get all premises of current argument
			List<ArgumentUnit> premises = new ArrayList<ArgumentUnit>();
			for (int premiseID : argument.getPremiseIDs()) {
				premises.add(allArgumentUnits.get(premiseID));
			}
			// Compute score
			argID2Score.put(argument.getArgumentID(), 
					this.computePremiseBasedScore(premises));
		}		
		// NOT USED (ONLY TESTED TO SEE EFFECTS FOR FUTURE RESEARCH)
//		argID2Score = this.applyTieBreakingRule(argID2Score, allArgumentUnits, 
//				allArguments);
		return new ArgumentRanking(conclusion.getArgumentUnitID(), argID2Score);
	}
	
	/**
	 * Applies a tie-breaking rule for all arguments in the given argument to
	 * score mapping that were assigned the same score by the ranker. In 
	 * particular, the longest premise is preferred then (measured in the number 
	 * of characters). For this purpose, access to the given sets of arguments
	 * and argument units is needed.
	 * 
	 * @param argID2Score Mapping from argument IDs to scores
	 * @param allArgumentUnits The set of all argument units
	 * @param allArguments The set of all arguments
	 * @return The strictly ordered ranking
	 */
	protected TreeMap<Integer, Double> applyTieBreakingRule(
			TreeMap<Integer, Double> argID2Score,
			Map<Integer, ArgumentUnit> allArgumentUnits,
			Map<Integer, Argument> allArguments){
		TreeMap<Integer, Double> finalArgID2Score = 
				new TreeMap<Integer, Double>();
		while (argID2Score.size() > 0){
			// Get maximum
			double max = -1.0;
			for (int argumentID : argID2Score.keySet()) {
				if (argID2Score.get(argumentID) > max){
					max = argID2Score.get(argumentID);
				}
			}
			// Find out how many best exist
			List<Integer> bestIDs = new ArrayList<Integer>();
			for (int argumentID : argID2Score.keySet()) {
				if (Double.compare(argID2Score.get(argumentID), max) == 0){
					bestIDs.add(argumentID);
				}
			}
			// Clear winner
			if (bestIDs.size() == 1){
				finalArgID2Score.put(bestIDs.get(0), 
						(double) argID2Score.size());
				argID2Score.remove(bestIDs.get(0));
			} 
			// Tie-breaking needed
			else{
				while (bestIDs.size() > 0){
					int maxLength = 0;
					int maxLengthIndex = -1;
					for (int i=0; i<bestIDs.size(); i++) {
						Argument argument = allArguments.get(bestIDs.get(i));
						int premiseLength = 0;
						for (int premiseID : argument.getPremiseIDs()) {
							ArgumentUnit premise = 
									allArgumentUnits.get(premiseID);
							premiseLength += premise.getText().length();
						}
						if (premiseLength > maxLength){
							maxLength = premiseLength;
							maxLengthIndex = i;
						}
					}
					finalArgID2Score.put(bestIDs.get(maxLengthIndex), 
							(double) argID2Score.size());
					argID2Score.remove(bestIDs.get(maxLengthIndex));
					bestIDs.remove(maxLengthIndex);
				}
			}
		}
		return finalArgID2Score;
	}
	
	
	// -------------------------------------------------------------------------
	// ABSTRACT METHOD OF THIS CLASS
	// -------------------------------------------------------------------------
	
	/**
	 * Computes an argument score based on the given premises.
	 * 
	 * @param premises The premises
	 * @return The argument score
	 */
	public abstract double computePremiseBasedScore(
			List<ArgumentUnit> premises);
}










