package argumentrelevance.argumentranker;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

import argumentrelevance.storage.Argument;
import argumentrelevance.storage.ArgumentRanking;
import argumentrelevance.storage.ArgumentUnit;

/**
 * This class is the abstract superclass of all evaluated ranking approaches
 * and baselines.
 * 
 * The ranker always creates real ranks, i.e., the resulting rank values will be 
 * 1, 2, ... In case two arguments have the same computed score, they will 
 * get the same rank (but the next rank is skipped then).
 * 
 * @author henning.wachsmuth
 *
 */
public abstract class AbstractRanker {

	/**
	 * Computes argument rankings for all conclusions in the given ground-truth
	 * ranking data. May access all given argument units and arguments. 
	 * 
	 * @param groundTruthRankingMap The ground-truth data
	 * @param allArgumentUnits The argument units
	 * @param allArguments The arguments
	 * @return The rankings
	 */
	public Map<Integer, ArgumentRanking> computeAllRankings(
			Map<Integer, List<ArgumentRanking>> groundTruthRankingMap,
			Map<Integer, ArgumentUnit> allArgumentUnits,
			Map<Integer, Argument> allArguments){
		Map<Integer, ArgumentRanking> rankings = 
				new HashMap<Integer, ArgumentRanking>();
		for (int conclusionID : groundTruthRankingMap.keySet()) {
			ArgumentRanking ranking = this.computeRanking(
					groundTruthRankingMap.get(conclusionID).get(0), 
					allArgumentUnits, allArguments);
			rankings.put(conclusionID, ranking);
		}
		return rankings;
	}
	
	/**
	 * Computes a ranking for the arguments in the given grount-truth ranking, 
	 * using information about any from the sets of all given argument units and
	 * arguments. 
	 * 
	 * @param groundTruthRanking The ground-truth ranking
	 * @param allArgumentUnits The argument units
	 * @param allArguments The arguments
	 * @return The ranking
	 */
	private ArgumentRanking computeRanking(
			ArgumentRanking groundTruthRanking,
			Map<Integer, ArgumentUnit> allArgumentUnits,
			Map<Integer, Argument> allArguments){
		// Get conclusion and all arguments that refer to conclusion
		ArgumentUnit conclusion = 
				allArgumentUnits.get(groundTruthRanking.getConclusionID());
		List<Argument> arguments = new ArrayList<Argument>();
		for (int argumentID : groundTruthRanking.getArgumentID2Rank().keySet()){
			arguments.add(allArguments.get(argumentID));
		}
		// Compute score ranking
		ArgumentRanking ranking = this.computeArgumentScores(conclusion, 
				arguments, allArgumentUnits, allArguments);
		// Convert scores to real ranks
		ranking = this.convertArgumentScoresToRanks(ranking);		
		return ranking;
	}
	
	/**
	 * Derives an empty ranking from the given ground-truth ranking. 
	 * An empty ranking contains all arguments associated to one conclusion, but 
	 * with NULL as the ranking value.
	 * 
	 * @param groundTruthRanking The ground-truth ranking
	 * @return The empty ranking
	 */
	public ArgumentRanking createEmptyRanking(
			ArgumentRanking groundTruthRanking){
		TreeMap<Integer, Double> emptyArgumentID2Rank = 
				new TreeMap<Integer, Double>();
		for (int argumentID : groundTruthRanking.getArgumentID2Rank().keySet()){
			emptyArgumentID2Rank.put(argumentID, null);
		}
		return new ArgumentRanking(groundTruthRanking.getConclusionID(), 
				emptyArgumentID2Rank);
	}
	
	/**
	 * Computes a ranking for the given arguments that refer to the given 
	 * conclusion, using  information about any from the sets of all given 
	 * argument units and arguments.
	 * 
	 * @param conclusion The conclusion
	 * @param arguments The arguments
	 * @param allArgumentUnits The argument units
	 * @param allArguments The arguments
	 * @return The ranking
	 */
	public abstract ArgumentRanking computeArgumentScores(
			ArgumentUnit conclusion, List<Argument> arguments,
			Map<Integer, ArgumentUnit> allArgumentUnits,
			Map<Integer, Argument> allArguments);
	
	/**
	 * Assumes that the rank values of the given ranking are scores (also works
	 * if they are real ranks themselves). Replaces the scores by real ranks,
	 * always assuming that higher scores are better.
	 * 
	 * @param scoreRanking The score ranking
	 * @return The real ranking
	 */
	public ArgumentRanking convertArgumentScoresToRanks(
			ArgumentRanking scoreRanking){	
		// Sort scores of input ranking
		Map<Integer, Double> argID2Rank = scoreRanking.getArgumentID2Rank();
		List<Double> scores = new ArrayList<Double>();
		for (int argumentID : argID2Rank.keySet()) {
			scores.add(argID2Rank.get(argumentID));
		}
		Collections.sort(scores);
		Collections.reverse(scores);
		// Now create new ranking with sorting ranks as values
		TreeMap<Integer, Double> sortedArgID2Rank = 
				new TreeMap<Integer, Double>();
		for (int argumentID : argID2Rank.keySet()) {
			double score = argID2Rank.get(argumentID);
			// Find rank and set
			for (int i=1; i<=scores.size(); i++){
				if (Double.compare(score, scores.get(i-1)) == 0){
					sortedArgID2Rank.put(argumentID, (double) i);
					break;
				}
			}
		}		
		return new ArgumentRanking(scoreRanking.getConclusionID(), 
				sortedArgID2Rank);
	}
}




