package argumentrelevance.argumentranker;

import java.io.BufferedReader;
import java.io.FileReader;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import argumentrelevance.storage.ArgumentUnit;

/**
 * This class is the abstract superclass of all evaluated ranking approaches
 * that compute an argument's score based on the SentiWordNet 1st sense 
 * sentiment of an argument's premises.
 * 
 * @author henning.wachsmuth
 *
 */
public abstract class AbstractSentiwordnetBasedRanker 
extends AbstractPremiseBasedRanker {
	
	// -------------------------------------------------------------------------
	// CONSTANTS
	// -------------------------------------------------------------------------
	
	/**
	 * Path of the SentiWordNet file
	 */
	private static final String SENTIWORDNET_PATH = 
			"conf/SentiWordNet_3.0.0_20130122.txt";
	
	
	// -------------------------------------------------------------------------
	// OBJECTS
	// -------------------------------------------------------------------------
	
	/**
	 * The SentiWordNet positivity scores. Loaded in the constructor.
	 */
	protected Map<String, Double> sentiWordNetPositives;
	
	/**
	 * The SentiWordNet negativity scores. Loaded in the constructor.
	 */
	protected Map<String, Double> sentiWordNetNegatives;
	
	
	
	// -------------------------------------------------------------------------
	// ADDITIONAL CONSTRUCTOR
	// -------------------------------------------------------------------------
	
	/**
	 * Loads the SentiWordNet 1st sense scores from the globally specified
	 * file path.
	 */
	public AbstractSentiwordnetBasedRanker() {
		super();
		sentiWordNetPositives = new HashMap<String, Double>();
		sentiWordNetNegatives = new HashMap<String, Double>();
		try{
			try (BufferedReader csv =  new BufferedReader(
					new FileReader(SENTIWORDNET_PATH));) {
				String line = "";	
				while((line = csv.readLine()) != null){
					if (line.charAt(0) == '#' || line.charAt(0) == '\t'){
						continue;
					}
					// Get scores of current words
					String[] data = line.split("\t");
					double posScore = Double.parseDouble(data[2]);
					double negScore = Double.parseDouble(data[3]);
					// Split synonym words
					String [] wordsWithNumbers = data[4].split(" ");
					for(String wordWithNumber: wordsWithNumbers){
						String [] parts = wordWithNumber.split("#");
						if (!"1".equals(parts[1]))
							continue;
						String word = parts[0].toLowerCase();
						this.sentiWordNetPositives.put(word, posScore);
						this.sentiWordNetNegatives.put(word, negScore);
					}
				}
			}
		} catch(Exception e){
			e.printStackTrace();
		}			
	}
	
	// -------------------------------------------------------------------------
	// IMPLEMENTATION OF SUPERCLASS METHOD
	// -------------------------------------------------------------------------
	
	@Override
	public double computePremiseBasedScore(List<ArgumentUnit> premises) {
		List<Double> sentiments = new ArrayList<Double>();
		for (ArgumentUnit premise : premises) {
			double sentiment = 0.0;
			Map<String, Integer> words = this.getWords(premise.getText());
			for (String word : words.keySet()) {
				if (sentiWordNetPositives.containsKey(word)){
					sentiment += 
						words.get(word) * sentiWordNetPositives.get(word);
				}
				if (sentiWordNetNegatives.containsKey(word)){
					sentiment -= 
							words.get(word) * sentiWordNetNegatives.get(word);
				}
			}
			sentiments.add(sentiment);
		}
		return this.computeSentimentBasedScore(sentiments);
	}
	
	
	/**
	 * Returns counts of all words contained in the given text span.
	 * 
	 * @param textSpan The text span
	 * @return The counts
	 */
	private Map<String, Integer> getWords(String textSpan){
		// Not case-sensitive
		textSpan = textSpan.toLowerCase();
		
		// Delete common non-word characters
		textSpan = textSpan.replace(",", "");
		textSpan = textSpan.replace(".", "");
		textSpan = textSpan.replace("!", "");
		textSpan = textSpan.replace("?", "");
		textSpan = textSpan.replace(":", "");
		textSpan = textSpan.replace(";", "");
		
		// Then split into words and count
		Map<String, Integer> counts = new HashMap<String, Integer>();
		for (String word : textSpan.split(" |\t|\n")) {
			if (!counts.containsKey(word)){
				counts.put(word, 0);
			}
			counts.put(word, counts.get(word)+1);
		}
		return counts;
	}
	
	
	// -------------------------------------------------------------------------
	// ABSTRACT METHODS OF THIS CLASS
	// -------------------------------------------------------------------------
	
	/**
	 * Computes an argument score based on the given values, each respresenting
	 * the sentiment of one of the argument's premises.
	 * 
	 * @param sentiments The sentiments
	 * @return The argument score
	 */
	public abstract double computeSentimentBasedScore(List<Double> sentiments);
}












