package argumentrelevance.argumentranker;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

import argumentrelevance.storage.Argument;
import argumentrelevance.storage.ArgumentRanking;
import argumentrelevance.storage.ArgumentUnit;

/**
 * This class is the abstract superclass of all evaluated ranking approaches
 * that compute an argument's score based on the similarity between
 * an argument's premises and its conclusion.
 * 
 * @author henning.wachsmuth
 *
 */
public abstract class AbstractSimilarityBasedRanker extends AbstractRanker {
	
	@Override
	public ArgumentRanking computeArgumentScores(
			ArgumentUnit conclusion, List<Argument> arguments,
			Map<Integer, ArgumentUnit> allArgumentUnits,
			Map<Integer, Argument> allArguments) {
		// Compute and return one score for each argument
		TreeMap<Integer, Double> argID2Score = new TreeMap<Integer, Double>();
		for (Argument argument : arguments) {
			// Get all premises of current argument
			List<ArgumentUnit> premises = new ArrayList<ArgumentUnit>();
			for (int premiseID : argument.getPremiseIDs()) {
				premises.add(allArgumentUnits.get(premiseID));
			}
			// Compute similarities and derive argument score from it
			List<Double> similarities = computeSimilarities(premises, 
					conclusion);
			argID2Score.put(argument.getArgumentID(), 
					this.computeSimilarityBasedScore(similarities));
		}		
		return new ArgumentRanking(conclusion.getArgumentUnitID(), argID2Score);	
	}
	
	
	// -------------------------------------------------------------------------
	// ABSTRACT METHODS OF THIS CLASS
	// -------------------------------------------------------------------------
	
	/**
	 * Computes an argument score based on the given values, each respresenting
	 * the similarity between the argument's conclusion and one of the premises.
	 * 
	 * @param similarities The similarities
	 * @return The argument score
	 */
	public abstract double computeSimilarityBasedScore(
			List<Double> similarities);
	
	/**
	 * Computes and returns one value for each given premise that captures the
	 * similarity between the premise and the given conclusion.
	 * 
	 * @param premises The premises
	 * @param conclusion The conclusion
	 * @return The similarity values
	 */
	public abstract List<Double> computeSimilarities(
			List<ArgumentUnit> premises, ArgumentUnit conclusion);

}


















