package argumentrelevance.rankingcorrelation;

import java.util.Map;
import java.util.TreeMap;

import argumentrelevance.storage.ArgumentRanking;

/**
 * This class is the abstract superclass of all classes that compute 
 * correlations between argument rankings.
 * 
 * @author henning.wachsmuth
 *
 */
public abstract class AbstractCorrelationComputer {

	/**
	 * Computes and returns one correlation value for each pair of associated
	 * argument rankings from the given two ranking maps.
	 * 
	 * @param rankingsA Rankings A
	 * @param rankingsB Rankings B
	 * @return The correlation values
	 */
	public TreeMap<Integer, Double> computeAllCorrelations(
			Map<Integer, ArgumentRanking> rankingsA,
			Map<Integer, ArgumentRanking> rankingsB){
		TreeMap<Integer, Double> correlations = new TreeMap<Integer, Double>();
		for (Integer conclusionID : rankingsA.keySet()) {
			Map<Integer,Double> argumentID2RankA = 
					rankingsA.get(conclusionID).getArgumentID2Rank();
			Map<Integer,Double> argumentID2RankB = 
					rankingsB.get(conclusionID).getArgumentID2Rank();
			correlations.put(conclusionID, this.computeCorrelation(
					argumentID2RankA, argumentID2RankB));
		}
		return correlations;
	}
	
	/**
	 * Computes the average correlation from the given correlations.
	 * 
	 * @param correlations The correlations
	 * @return The average correlation
	 */
	public double computeAverageCorrelation(Map<Integer, Double> correlations){
		double average = 0.0;
		for (int conclusionID : correlations.keySet()) {
			average += correlations.get(conclusionID);
		}
		if (correlations.size() > 0){
			return average /= correlations.size();
		}
		return 0.0;
	}
	
	/**
	 * Computes the correlation between the two given argument rankings.
	 * @param argumentID2RankA Ranking A
	 * @param argumentID2RankB Ranking B
	 * @return The correlation
	 */
	public abstract double computeCorrelation(
			Map<Integer,Double> argumentID2RankA, 
			Map<Integer,Double> argumentID2RankB);
}


























