package argumentrelevance.storage;

import java.util.List;

/**
 * Storage class for all information of an argument unit.
 * 
 * @author henning.wachsmuth
 *
 */
public class ArgumentUnit {

	// -------------------------------------------------------------------------
	// VALUES
	// -------------------------------------------------------------------------

	/**
	 * The global ID of the argument unit.
	 */
	private int argumentUnitID;
	
	/**
	 * The text of the argument unit.
	 */
	private String text;
	
	/**
	 * The corrected version of the argument unit text or NULL if no corrected 
	 * version is known.
	 */
	private String correctedText;
	
	/**
	 * The absolute frequence of the argument unit text in the AIFdb corpora.
	 */
	private int frequency;
	
	/**
	 * The computed PageRank value of the argument unit
	 */
	private double pageRank;
	
	/**
	 * All source information of the argument unit
	 */
	private List<ArgumentUnitSource> sources;


	
	// -------------------------------------------------------------------------
	// INITIALIZATION
	// -------------------------------------------------------------------------

	/**
	 * Sets all values.
	 * 
	 * @param ID The ID
	 * @param text The text
	 * @param correctedText The corrected text or NULL
	 * @param frequency The absolute frequence
	 * @param pageRank The PageRank value
	 * @param sources All source information
	 */
	public ArgumentUnit(int ID, String text, String correctedText,
			int frequency, double pageRank, List<ArgumentUnitSource> sources) {
		super();
		this.argumentUnitID = ID;
		this.text = text;
		this.correctedText = correctedText;
		this.frequency = frequency;
		this.pageRank = pageRank;
		this.sources = sources;
	}


	
	// -------------------------------------------------------------------------
	// GETTERS AND SETTERS
	// -------------------------------------------------------------------------

	/**
	 * Gets the ID.
	 * @return The ID
	 */
	public int getArgumentUnitID() {
		return argumentUnitID;
	}

	/**
	 * Sets the ID
	 * @param ID The ID
	 */
	public void setArgumentUnitID(int ID) {
		this.argumentUnitID = ID;
	}

	/**
	 * Gets the text.
	 * @return The text.
	 */
	public String getText() {
		return text;
	}

	/**
	 * Sets the text.
	 * @param text The text.
	 */
	public void setText(String text) {
		this.text = text;
	}

	/**
	 * Gets the corrected text.
	 * @return The corrected text.
	 */
	public String getCorrectedText() {
		return correctedText;
	}

	/**
	 * Sets the corrected text.
	 * @param correctedText The corrected text
	 */
	public void setCorrectedText(String correctedText) {
		this.correctedText = correctedText;
	}

	/**
	 * Gets the absolute frequency.
	 * @return The absolute frequency
	 */
	public int getFrequency() {
		return frequency;
	}

	/**
	 * Sets the absolute frequency.
	 * @param frequency The absolute frequency.
	 */
	public void setFrequency(int frequency) {
		this.frequency = frequency;
	}

	/**
	 * Gets the PageRank value.
	 * @return The PageRank value.
	 */
	public double getPageRank() {
		return pageRank;
	}

	/**
	 * Sets the PageRank value.
	 * @param pageRank The PageRank value.
	 */
	public void setPageRank(double pageRank) {
		this.pageRank = pageRank;
	}

	/**
	 * Gets all sources.
	 * @return The sources.
	 */
	public List<ArgumentUnitSource> getSources() {
		return sources;
	}

	/**
	 * Sets all sources.
	 * @param sources The sources
	 */
	public void setSources(List<ArgumentUnitSource> sources) {
		this.sources = sources;
	}
}












