package argumentrelevance.storage;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

/**
 * Loader for the CSV files used in the ranking
 * 
 * @author henning.wachsmuth
 *
 */
public class CSVLoader {
	
	/**
	 * Loads the unit sources from the specified input file and
	 * returns them in a map.
	 * 
	 * @param unitSourcePath The path of the input file
	 * @return The unit source map
	 */
	public Map<Integer, List<ArgumentUnitSource>> loadUnitSourceMap(
			String unitSourcePath){
		Map<Integer, List<ArgumentUnitSource>> unitSourceMap = 
				new HashMap<Integer, List<ArgumentUnitSource>>();
		System.out.print("Loading unit sources...");
		int loaded = 0;
		try{
			BufferedReader reader = new BufferedReader(new InputStreamReader(
					new FileInputStream(unitSourcePath), "utf-8"));	
			// Skip title line by reading it now
			String line = reader.readLine();

			// Create one map entry for each other line
			while ((line = reader.readLine()) != null){
				// Get source information
				String [] allValues = line.split(",");
				int argumentUnitID = Integer.parseInt(allValues[0]);
				ArgumentUnitSource source = new ArgumentUnitSource(
						Integer.parseInt(allValues[2]), allValues[1], 
						Integer.parseInt(allValues[3]));
				// Add to list that refers to ID
				if (!unitSourceMap.containsKey(argumentUnitID)){
					unitSourceMap.put(argumentUnitID, 
							new ArrayList<ArgumentUnitSource>());
				}
				unitSourceMap.get(argumentUnitID).add(source);
				loaded++;
				if (loaded % 1000 == 0){
					System.out.print(".");
				}
			}
			// Close file
			reader.close();
		} catch(Exception ex){
			System.out.println("failed at unit source " + (loaded+1) + ".");
			ex.printStackTrace();
		}

		System.out.println(unitSourceMap.size() + " sources loaded.");
		return unitSourceMap;
	}
	
	/**
	 * Loads all argument units from the specified input file and
	 * returns them in a map. Adds the given unit sources to the respective 
	 * units.
	 * 
	 * @param argumentUnitPath The path of the input file
	 * @param unitSourceMap The unit source map
	 * @return The argument unit map
	 */
	public Map<Integer, ArgumentUnit> loadArgumentUnitMap(
			String argumentUnitPath,
			Map<Integer, List<ArgumentUnitSource>> unitSourceMap){
		Map<Integer, ArgumentUnit> argumentUnitMap = 
				new HashMap<Integer,ArgumentUnit>();
		System.out.print("Loading argument units...");
		int loaded = 0;
		try{
			BufferedReader reader = new BufferedReader(new InputStreamReader(
					new FileInputStream(argumentUnitPath), "utf-8"));	
			// Skip title line by reading it now
			String line = reader.readLine();

			// Create one map entry for each other line
			while ((line = reader.readLine()) != null){
				int startOfText = line.indexOf("\"");
				int endOfText = line.lastIndexOf("\"");
				int argumentUnitID = 
						Integer.parseInt(line.substring(0, startOfText-1));
				String text = line.substring(startOfText+1, endOfText);
				text = text.replaceAll("&quot;", "\"");
				text = text.replaceAll("&#10", "\n");
				String [] freqAndRank = line.substring(endOfText+2).split(",");
				ArgumentUnit argumentUnit = new ArgumentUnit(
						argumentUnitID, text, null, 
						Integer.parseInt(freqAndRank[0]), 
						Double.parseDouble(freqAndRank[1]), 
						unitSourceMap.get(argumentUnitID));
				argumentUnitMap.put(argumentUnitID, argumentUnit);
				// TEST OUTPUT
//				System.out.println("Argument unit " + argumentUnitID 
//						+ " has frequency " + freqAndRank[0]
//						+ " and PageRank " + freqAndRank[1] + ".");
				
				loaded++;
				if (loaded % 1000 == 0){
					System.out.print(".");
				}
			}
			// Close file
			reader.close();
		} catch(Exception ex){
			System.out.println("failed at argument unit " + (loaded+1) + ".");
			ex.printStackTrace();
		}
		System.out.println(argumentUnitMap.size() + " units loaded.");
		return argumentUnitMap;
	}
	
	/**
	 * Loads all arguments from the specified input file and
	 * returns them in a map. 
	 * 
	 * @param argumentPath The path of the input file
	 * @return The argument map
	 */
	public Map<Integer, Argument> loadArgumentMap(String argumentPath){
		Map<Integer, Argument> argumentMap = new HashMap<Integer,Argument>();
		System.out.print("Loading arguments...");
		int loaded = 0;
		try{
			BufferedReader reader = new BufferedReader(new InputStreamReader(
					new FileInputStream(argumentPath), "utf-8"));	
			// Skip title line by reading it now
			String line = reader.readLine();

			// Create one map entry for each other line
			while ((line = reader.readLine()) != null){
				int startOfConclusion = line.indexOf("\"");
				String [] argIDAndConcID = 
						line.substring(0, startOfConclusion-1).split(",");
				int argumentID = Integer.parseInt(argIDAndConcID[0]);
				int conclusionID = Integer.parseInt(argIDAndConcID[1]);
				String rest = line.substring(startOfConclusion+1);
				rest = rest.substring(rest.indexOf("\"")+2);
				String [] restSplit = rest.split(",");
				int numberOfPremises = Integer.parseInt(restSplit[0]);
				rest = rest.substring(restSplit[0].length()+1);
				List<Integer> premiseIDs = new ArrayList<Integer>();
				for (int i=0; i<numberOfPremises; i++){
					restSplit = rest.split(",");
					premiseIDs.add(Integer.parseInt(restSplit[0]));
					rest = rest.substring(restSplit[0].length()+2);
					if (i<numberOfPremises-1){
						rest = rest.substring(rest.indexOf("\"")+2);
					}
				}
				Argument argument = 
						new Argument(argumentID, conclusionID, premiseIDs);
				argumentMap.put(argumentID, argument);
//				// TEST OUTPUT
//				System.out.println("Argument " + argumentID + " has conclusion " 
//						+ conclusionID + " and " + premiseIDs.size() 
//						+ " premises.");
				loaded++;
				if (loaded % 1000 == 0){
					System.out.print(".");
				}
			}
			// Close file
			reader.close();
		} catch(Exception ex){
			System.out.println("failed at argument " + (loaded+1) + ".");
			ex.printStackTrace();
		}
		System.out.println(argumentMap.size() + " arguments loaded.");
		return argumentMap;
	}
	
	/**
	 * Loads all ground-truth argument rankings from the specified 
	 * input file, for which it needs the given number of annotators. Returns 
	 * the rankings in a map. 
	 * 
	 * @param groundTruthPath The path of the input file
	 * @param numberOfAnnotators The number of annotators
	 * @return The rankings
	 */
	public Map<Integer, List<ArgumentRanking>> loadAllGroundTruthRankings(
			String groundTruthPath, int numberOfAnnotators){
		Map<Integer, List<ArgumentRanking>> conclusionID2Rankings = 
				new HashMap<Integer, List<ArgumentRanking>>();
		System.out.print("Loading ground-truth argument rankings...");
		int loaded = 0;
		try{
			BufferedReader reader = new BufferedReader(new InputStreamReader(
					new FileInputStream(groundTruthPath), "utf-8"));	
			// Skip title line by reading it now
			String line = reader.readLine();

			// Create one map entry for each other line
			while ((line = reader.readLine()) != null){
				// Get all information from line
				String [] allValues = line.split(",");
				int argumentID = Integer.parseInt(allValues[0]);
				List<Double> ranks = new ArrayList<Double>();
				for (int i=1; i<=numberOfAnnotators+2; i++){
					ranks.add(Double.parseDouble(allValues[i]));
				}
				int conclusionID = 
						Integer.parseInt(allValues[numberOfAnnotators+3]);
				
				// Create all rankings for conclusion if not yet existing
				if (!conclusionID2Rankings.containsKey(conclusionID)){
					List<ArgumentRanking> rankings = 
							new ArrayList<ArgumentRanking>();
					for (int i=0; i<numberOfAnnotators+2; i++){
						rankings.add(new ArgumentRanking(conclusionID, 
								new TreeMap<Integer, Double>()));
					}
					conclusionID2Rankings.put(conclusionID, rankings);
//					// TEST OUTPUT
//					System.out.println("Conclusion " + conclusionID + " has " 
//							+ rankings.size() + " rankings.");
				}
				
				// Now add ranks of current argument to all rankings
				List<ArgumentRanking> rankings = 
						conclusionID2Rankings.get(conclusionID);
				for (int i=0; i<numberOfAnnotators+2; i++){
					rankings.get(i).getArgumentID2Rank().put(argumentID, 
							ranks.get(i));
				}
				
				loaded++;
				if (loaded % 1000 == 0){
					System.out.print(".");
				}
			}
			// Close file
			reader.close();
		} catch(Exception ex){
			System.out.println("failed at ranking " + (loaded+1) + ".");
			ex.printStackTrace();
		}
		System.out.println(conclusionID2Rankings.size() + " rankings loaded.");
		return conclusionID2Rankings;
	}
	

	
}











































